@@ -98,6 +98,7 @@ DataBackendDuckDB = R6Class("DataBackendDuckDB", inherit = DataBackend, cloneabl
       assert_choice(data_format, self$data_formats)
       cols = intersect(cols, self$colnames)
       tmp_tbl = write_temp_table(private$.data, rows)
+      on.exit(DBI::dbRemoveTable(private$.data, tmp_tbl, temporary = TRUE))
 
       query = sprintf('SELECT %1$s FROM "%2$s" INNER JOIN "%3$s" ON "%2$s"."row_id" = "%3$s"."%4$s"',
         paste0(sprintf('"%s"."%s"', self$table, union(cols, self$primary_key)), collapse = ","),
@@ -133,6 +134,7 @@ DataBackendDuckDB = R6Class("DataBackendDuckDB", inherit = DataBackend, cloneabl
       private$.reconnect()
       assert_names(cols, type = "unique")
       cols = intersect(cols, self$colnames)
+      order = sprintf('ORDER BY "%s"', self$primary_key)
 
       if (is.null(rows)) {
         get_query = function(col) {
@@ -140,6 +142,7 @@ DataBackendDuckDB = R6Class("DataBackendDuckDB", inherit = DataBackend, cloneabl
         }
       } else {
         tmp_tbl = write_temp_table(private$.data, rows)
+        on.exit(DBI::dbRemoveTable(private$.data, tmp_tbl, temporary = TRUE))
 
         get_query = function(col) {
           sprintf('SELECT DISTINCT("%1$s"."%2$s") FROM "%3$s" LEFT JOIN "%1$s" ON "%3$s"."row_id" = "%1$s"."%4$s"',
@@ -152,7 +155,7 @@ DataBackendDuckDB = R6Class("DataBackendDuckDB", inherit = DataBackend, cloneabl
         if (na_rm) {
           query = sprintf('%s WHERE "%s"."%s" IS NOT NULL', query, self$table, col)
         }
-        levels = DBI::dbGetQuery(private$.data, query)[[1L]]
+        levels = DBI::dbGetQuery(private$.data, paste(query, order))[[1L]]
         if (is.factor(levels)) as.character(levels) else levels
       })
 
@@ -175,6 +178,7 @@ DataBackendDuckDB = R6Class("DataBackendDuckDB", inherit = DataBackend, cloneabl
       }
 
       tmp_tbl = write_temp_table(private$.data, rows)
+      on.exit(DBI::dbRemoveTable(private$.data, tmp_tbl, temporary = TRUE))
 
       query = sprintf('SELECT %1$s FROM (SELECT * FROM "%2$s" INNER JOIN "%3$s" ON "%2$s"."%4$s" = "%3$s"."row_id")',
         paste0(sprintf('COUNT("%s")', cols), collapse = ","),
